import { Model, SDK, Response } from './common';

module Team {

    export interface TeamInterface {

        id?: number;
        event_id: number;
        team_number?: number;
        name: string;
        repo?: string;
        devpost_url?: string;
        channel_id?: string;
    }

    interface CreateTeamResponse extends Response {

        team: TeamInterface;
    }

    interface GetTeamByIdResponse extends Response {

        team: TeamInterface;
    }

    interface GetAllTeamResponse extends Response {

        teams: TeamInterface[];
    }

    interface GetAttendeesResponse extends Response {

        attendees: any[];
    }

    interface GetEventResponse extends Response {

        event: any;
    }

    export async function create(team: TeamInterface): Promise<Instance> {
        try {
            let res = <CreateTeamResponse>await SDK.getInstance().POST("team", team);
            return new Instance(res.team);
        } catch (err) {
            return null;
        }
    }

    export async function getById(id: number): Promise<Instance> {
        try {
            let res = <GetTeamByIdResponse>await SDK.getInstance().GET("team/" + String(id));
            if (res.success && res.team) {
                return new Instance(res.team);
            } else {
                return null;
            }
        } catch (err) {
            return null;
        }
    }

    export async function getAll(): Promise<Instance[]> {
        try {
            let res = <GetAllTeamResponse>await SDK.getInstance().GET("team");
            if (res.success && res.teams) {
                let teams: Instance[] = [];
                for (let event of res.teams) {
                    teams.push(new Instance(event));
                }
                return teams;
            } else {
                return null;
            }
        } catch (err) {
            return null;
        }
    }

    export class Instance implements TeamInterface, Model {

        id?: number;
        event_id: number;
        team_number?: number;
        name: string;
        repo?: string;
        devpost_url?: string;
        channel_id?: string;

        constructor(team: TeamInterface) {

            this.id = team.id;
            this.event_id = team.event_id;
            this.team_number = team.team_number;
            this.name = team.name;
            this.repo = team.repo;
            this.devpost_url = team.devpost_url;
            this.channel_id = team.channel_id;
        }

        public async save(): Promise<boolean> {
            return new Promise<boolean>(async (resolve, reject) => {
                let result = <Response>await SDK.getInstance().PUT("team/" + String(this.id), {
                    event_id: this.event_id,
                    team_number: this.team_number,
                    name: this.name,
                    repo: this.repo,
                    devpost_url: this.devpost_url,
                    channel_id: this.channel_id
                });
                if (result.success) {
                    resolve(true);
                } else {
                    reject(result.err);
                }
            });
        }

        public async delete(): Promise<boolean> {
            return new Promise<boolean>(async (resolve, reject) => {
                let result = <Response>await SDK.getInstance().DELETE("team/" + String(this.id), {});
                if (result.success) {
                    resolve(true);
                } else {
                    reject(result.err);
                }
            });
        }

        public async getAttendees() {
            try {
                let result = <GetAttendeesResponse>await SDK.getInstance()
                    .GET("team/" + String(this.id) + "/attendee");
                if (result.success && result.attendees) {
                    return result.attendees;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }

        public async getEvent() {
            try {
                let result = <GetEventResponse>await SDK.getInstance()
                    .GET("team/" + String(this.id) + "/event");
                if (result.success && result.event) {
                    return result.event;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }
    }
}
export = Team;
