import { expect } from 'chai';
import { Event, Auth } from '../index';

describe('Event functions test', () => {

    let event: Event.Instance;

    before(() => {
        return Auth.login({
            username: process.env.USERNAME,
            password: process.env.PASSWORD,
        });
    });

    it('Should pass if GetAll returns an array of Event', () => {
        return Event.getAll().then((events: Event.Instance[]) => {
            expect(events).to.not.equal(null);
        });
    });

    it('Should pass if Create returns an Event', () => {
        return Event.create({
            name: "EventTest",
            image: "http://",
            begin_date: new Date(),
            end_date: new Date()
        }).then((newEvent: Event.Instance) => {
            event = newEvent;
            expect(event).to.not.equal(null);
        });
    });

    it('Should pass if GetById returns an array of Event', () => {
        return Event.getById(event.id).then((e: Event.Instance) => {
            expect(e).to.not.equal(null);
        });
    });

    it('Should pass if Update returns true', () => {
        event.image = "http://new_image";
        return event.save().then((success) => {
            expect(success).to.equal(true);
            return Event.getById(event.id).then((e: Event.Instance) => {
                expect(e).to.not.equal(null);
                expect(e.image).to.equal("http://new_image");
            });
        });
    });

    it('Should pass if GetActivities returns an array of Event', () => {
        return event.getActivities().then((activities) => {
            expect(activities).to.not.equal(null);
        });
    });

    it('Should pass if GetAttendees returns an array of Event', () => {
        return event.getAttendees().then((attendees) => {
            expect(attendees).to.not.equal(null);
        });
    });

    it('Should pass if GetForms returns an array of Event', () => {
        return event.getForms().then((forms) => {
            expect(forms).to.not.equal(null);
        });
    });

    it('Should pass if GetPartners returns an array of Event', () => {
        return event.getPartners().then((partners) => {
            expect(partners).to.not.equal(null);
        });
    });

    it('Should pass if GetSponsors returns an array of Event', () => {
        return event.getSponsors().then((sponsors) => {
            expect(sponsors).to.not.equal(null);
        });
    });

    it('Should pass if GetSponsorshipTiers returns an array of Event', () => {
        return event.getSponsorshipTiers().then((sponsorshipTier) => {
            expect(sponsorshipTier).to.not.equal(null);
        });
    });

    it('Should pass if GetTeams returns an array of Event', () => {
        return event.getTeams().then((teams) => {
            expect(teams).to.not.equal(null);
        });
    });

    it('Should pass if GetVolunteers returns an array of Event', () => {
        return event.getVolunteers().then((volunteers) => {
            expect(volunteers).to.not.equal(null);
        });
    });

    it('Should pass if Delete returns true', () => {
        return event.delete().then((success) => {
            expect(success).to.equal(true);
        });
    });

    after(() => {
        return Auth.logout();
    });
});
