import { expect } from 'chai';
import { Team, Auth } from '../index';

describe('Team functions test', () => {

    let team: Team.Instance;

    before(() => {
        return Auth.login({
            username: process.env.USERNAME,
            password: process.env.PASSWORD,
        });
    });

    it('Should pass if GetAll returns an array of Team', () => {
        return Team.getAll().then((teams: Team.Instance[]) => {
            expect(teams).to.not.equal(null);
        });
    });

    it('Should pass if Create returns an Team', () => {
        return Team.create({
            name: "TeamTest",
            event_id: 1,
            team_number: 1
        }).then((newTeam: Team.Instance) => {
            team = newTeam;
            expect(team).to.not.equal(null);
        });
    });

    it('Should pass if GetById returns an array of Team', () => {
        return Team.getById(team.id).then((t: Team.Instance) => {
            expect(t).to.not.equal(null);
        });
    });

    it('Should pass if Update returns true', () => {
        team.channel_id = "CHANNEL_SLACK";
        return team.save().then((success) => {
            expect(success).to.equal(true);
            return Team.getById(team.id).then((t: Team.Instance) => {
                expect(t).to.not.equal(null);
                expect(t.channel_id).to.equal("CHANNEL_SLACK");
            });
        });
    });

    it('Should pass if GetAttendees returns an array of Team', () => {
        return team.getAttendees().then((attendees) => {
            expect(attendees).to.not.equal(null);
        });
    });

    it('Should pass if GetEvent returns an array of Team', () => {
        return team.getEvent().then((event) => {
            expect(event).to.not.equal(null);
        });
    });

    it('Should pass if Delete returns true', () => {
        return team.delete().then((success) => {
            expect(success).to.equal(true);
        });
    });

    after(() => {
        return Auth.logout();
    });
});
